/*
 *  Copyright (C) 2009  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <atompub/content.h>
#include <atompub/content-xml.h>

#include <glib.h>
#include <libxml/tree.h>
#include <string.h>

struct _atom_content
{
  char *type;
  char *src;
  char *content;
  size_t content_len;
  xmlNodePtr xmlcontent;
};

AtomContent *
atom_content_new (char *type, char *buffer, size_t len)
{
  AtomContent *content;
  content = g_slice_new0 (AtomContent);
  content->type = g_strdup (type);
  content->content = g_malloc (len);
  memcpy (content->content, buffer, len);
  content->content_len = len;
  return content;
}

AtomContent *
atom_content_new_src (char *type, char *src)
{
  AtomContent *content;
  content = g_slice_new0 (AtomContent);
  content->type = g_strdup (type);
  content->src = g_strdup (src);
  return content;
}

AtomContent *
atom_content_new_from_xmlnode (xmlNodePtr root)
{
  AtomContent *content;
  xmlNodePtr child;
  if (xmlStrcmp (root->name, "content"))
    {
      return NULL;
    }
  content = g_slice_new0 (AtomContent);
  if ((content->type = xmlGetProp (root, "type")) == NULL)
    {
      g_slice_free (AtomContent, content);
      return NULL;
    }
  content->src = xmlGetProp (root, "src");
  if (content->src == NULL)
    {
      /* libxml2 does not help a lot here. We must decide between a
	 XML child or a content child. Using the type attribute might
	 be a good idead. Let's just ask that the current method
	 works. */
      if (root->children && root->children->type == XML_ELEMENT_NODE)
	{
	  content->xmlcontent = xmlCopyNodeList (root->children);
	}
      else
	{
	  content->content = xmlNodeGetContent (root);
	  content->content_len = xmlStrlen (content->content);
	}
    }
  return content;
}

AtomContent *
atom_content_new_data_len (char * buffer, size_t len)
{
  AtomContent *content;
  xmlDocPtr doc;
  xmlNodePtr root;
  doc = xmlReadMemory (buffer, len, NULL, NULL,
		       XML_PARSE_RECOVER | XML_PARSE_NOERROR);
  if (doc == NULL || (root = xmlDocGetRootElement (doc)) == NULL)
    return NULL;
  content = atom_content_new_from_xmlnode (root);
  xmlFreeDoc (doc);
  return content;
}

void
atom_content_delete (AtomContent *content)
{
  if (content->type)
    g_free (content->type);
  if (content->src)
    g_free (content->src);
  if (content->content)
    g_free (content->content);
  if (content->xmlcontent)
    xmlFreeNode (content->xmlcontent);
  g_slice_free (AtomContent, content);
}

char *
atom_content_type (AtomContent *content)
{
  return content->type;
}

void
atom_content_type_set (AtomContent *content, char *type)
{
  if (content->type)
    g_free (content->type);
  content->type = g_strdup (type);
}

char *
atom_content_src (AtomContent *content)
{
  return content->src;
}

void
atom_content_src_set (AtomContent *content, char *src)
{
  if (content->src)
    g_free (content->src);
  if (content->content)
    {
      g_free (content->content);
      content->content = NULL;
    }
  if (content->xmlcontent)
    {
      xmlFreeNode (content->xmlcontent);
      content->xmlcontent = NULL;
    }
  content->src = g_strdup (src);
}

void
atom_content_content (AtomContent *content, char **buffer, size_t *len)
{
  if (content->content == NULL)
    {
      if (buffer)
	*buffer = NULL;
      if (len)
	*len = 0;
    }
  else
    {
      if (buffer)
	{
	  *buffer = g_malloc (content->content_len);
	  memcpy (*buffer, content->content, content->content_len);
	}
      if (len)
	*len = content->content_len;
    }
}

void
atom_content_content_set (AtomContent *content, char *buffer, size_t len)
{
  if (content->content)
    g_free (content->content);
  if (content->src)
    {
      g_free (content->src);
      content->src = NULL;
    }
  if (content->xmlcontent)
    {
      xmlFreeNode (content->xmlcontent);
      content->xmlcontent = NULL;
    }
  content->content = g_malloc (len);
  memcpy (content->content, buffer, len);
  content->content_len = len;
}

void
atom_content_string (AtomContent *content, char **buffer, size_t *len)
{
  xmlDocPtr doc;
  xmlNodePtr node;
  node = atom_content_to_xmlnode (content);
  if (node == NULL)
    {
      if (buffer)
	*buffer = NULL;
      if (len)
	*len = 0;
      return;
    }
  doc = xmlNewDoc ("1.0");
  xmlDocSetRootElement (doc, node);
  xmlDocDumpMemory (doc, buffer, len);
  xmlFreeDoc (doc);
}

AtomContent *
atom_content_new_xmlnode (char *type, xmlNodePtr node)
{
  AtomContent *content;
  content = g_slice_new0 (AtomContent);
  content->type = g_strdup (type);
  content->xmlcontent = xmlCopyNodeList (node);
  return content;
}

xmlNodePtr
atom_content_to_xmlnode (AtomContent *content)
{
  xmlNodePtr node;
  node = xmlNewNode (NULL, "content");
  xmlNewProp (node, "type", content->type);
  if (content->src)
    {
      xmlNewProp (node, "src", content->src);
      return node;
    }
  else if (content->content)
    {
      xmlNodeSetContentLen (node, content->content, content->content_len);
      return node;
    }
  else if (content->xmlcontent)
    {
      xmlAddChild (node, xmlCopyNodeList (content->xmlcontent));
      return node;
    }
  return node;
}

xmlNodePtr
atom_content_content_xmlnode (AtomContent *content)
{
  return content->xmlcontent;
}

void
atom_content_content_set_xmlnode (AtomContent *content, xmlNodePtr node)
{
  if (content->xmlcontent)
    xmlFreeNode (content->xmlcontent);
  if (content->content)
    {
      g_free (content->content);
      content->content = NULL;
    }
  if (content->src)
    {
      g_free (content->src);
      content->src = NULL;
    }
  content->xmlcontent = xmlCopyNodeList (node);
}
